# -*- coding: utf-8 -*-
#!/usr/bin/env python3

# pip install librosa
# pip install soundfile
# pip install flask-cors
# pip install flask

import librosa
import soundfile
import os
import tempfile
import logging
from werkzeug.utils import secure_filename
from flask import Flask, request, jsonify, send_file
from flask_cors import CORS
import traceback

# 配置日志
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

app = Flask(__name__)
# 简化的CORS配置
CORS(app, expose_headers=["Content-Disposition"])

class PitchShifter:
    def __init__(self):
        self.audio_data = None
        self.sample_rate = None
        
    def load_audio(self, input_path):
        try:
            logger.info(f"开始加载音频文件: {input_path}")
            if not os.path.exists(input_path):
                raise FileNotFoundError(f"找不到音频文件: {input_path}")
            
            self.audio_data, self.sample_rate = librosa.load(input_path)
            logger.info(f"音频文件加载成功，采样率: {self.sample_rate}")
            
        except Exception as e:
            logger.error(f"加载音频文件失败: {str(e)}")
            raise
        
    def shift_pitch(self, steps):
        try:
            logger.info(f"开始转调，步数: {steps}")
            if self.audio_data is None:
                raise ValueError("请先加载音频文件")
                
            shifted = librosa.effects.pitch_shift(
                self.audio_data, 
                sr=self.sample_rate, 
                n_steps=steps
            )
            logger.info("转调完成")
            return shifted
            
        except Exception as e:
            logger.error(f"转调失败: {str(e)}")
            raise
        
    def save_audio(self, output_path, shifted_audio):
        try:
            logger.info(f"开始保存音频到: {output_path}")
            soundfile.write(output_path, shifted_audio, self.sample_rate)
            logger.info("音频保存成功")
            
        except Exception as e:
            logger.error(f"保存音频失败: {str(e)}")
            raise

@app.route('/pyapi/test', methods=['GET'])
def test_api():
    return jsonify({'data': '测试test接口通过'})

@app.route('/pyapi/changePitch', methods=['POST', 'OPTIONS'])
def change_pitch():
    if request.method == 'OPTIONS':
        return '', 204
        
    try:
        logger.info("收到转调请求")
        
        # 检查是否有文件上传
        if 'file' not in request.files:
            logger.warning("没有上传文件")
            return jsonify({'error': '没有上传文件'}), 400
        
        file = request.files['file']
        if file.filename == '':
            logger.warning("未选择文件")
            return jsonify({'error': '未选择文件'}), 400
            
        # 获取转调步数
        steps = request.form.get('steps', type=float)
        if steps is None:
            logger.warning("未指定转调步数")
            return jsonify({'error': '未指定转调步数'}), 400

        logger.info(f"处理文件: {file.filename}, 转调步数: {steps}")

        # 创建临时目录存放文件
        with tempfile.TemporaryDirectory() as temp_dir:
            # 保存上传的文件
            input_filename = secure_filename(file.filename)
            input_path = os.path.join(temp_dir, input_filename)
            file.save(input_path)
            logger.info(f"文件已保存到临时目录: {input_path}")
            
            # 处理音频转调
            shifter = PitchShifter()
            shifter.load_audio(input_path)
            shifted_audio = shifter.shift_pitch(steps)
            
            # 保存转调后的音频，确保有正确的扩展名
            output_filename = f"shifted_{os.path.splitext(input_filename)[0]}.wav"
            output_path = os.path.join(temp_dir, output_filename)
            logger.info(f"准备保存音频到: {output_path}")
            shifter.save_audio(output_path, shifted_audio)
            
            logger.info(f"准备返回文件: {output_filename}")
            # 设置响应头
            response = send_file(
                output_path,
                as_attachment=True,
                attachment_filename=output_filename,
                mimetype='audio/wav'
            )
            response.headers['Access-Control-Expose-Headers'] = 'Content-Disposition'
            return response
            
    except Exception as e:
        error_msg = f"处理失败: {str(e)}\n{traceback.format_exc()}"
        logger.error(error_msg)
        return jsonify({
            'error': str(e),
            'detail': traceback.format_exc()
        }), 500

@app.route('/pyapi/changePitchMp3', methods=['POST', 'OPTIONS'])
def change_pitch_mp3():
    if request.method == 'OPTIONS':
        return '', 204
        
    try:
        logger.info("收到MP3转调请求")
        
        # 检查是否有文件上传
        if 'file' not in request.files:
            logger.warning("没有上传文件")
            return jsonify({'error': '没有上传文件'}), 400
        
        file = request.files['file']
        if file.filename == '':
            logger.warning("未选择文件")
            return jsonify({'error': '未选择文件'}), 400
            
        # 获取转调步数
        steps = request.form.get('steps', type=float)
        if steps is None:
            logger.warning("未指定转调步数")
            return jsonify({'error': '未指定转调步数'}), 400

        logger.info(f"处理文件: {file.filename}, 转调步数: {steps}")

        # 创建临时目录存放文件
        with tempfile.TemporaryDirectory() as temp_dir:
            # 保存上传的文件
            input_filename = secure_filename(file.filename)
            input_path = os.path.join(temp_dir, input_filename)
            file.save(input_path)
            logger.info(f"文件已保存到临时目录: {input_path}")
            
            # 处理音频转调
            shifter = PitchShifter()
            shifter.load_audio(input_path)
            shifted_audio = shifter.shift_pitch(steps)
            
            # 保存为MP3格式
            output_filename = f"shifted_{os.path.splitext(input_filename)[0]}.mp3"
            output_path = os.path.join(temp_dir, output_filename)
            logger.info(f"准备保存MP3文件到: {output_path}")
            
            # 直接保存为MP3
            soundfile.write(output_path, shifted_audio, shifter.sample_rate, format='mp3')
            
            logger.info(f"准备返回MP3文件: {output_filename}")
            # 设置响应头
            response = send_file(
                output_path,
                as_attachment=True,
                attachment_filename=output_filename,
                mimetype='audio/mpeg'
            )
            response.headers['Access-Control-Expose-Headers'] = 'Content-Disposition'
            return response
            
    except Exception as e:
        error_msg = f"处理失败: {str(e)}\n{traceback.format_exc()}"
        logger.error(error_msg)
        return jsonify({
            'error': str(e),
            'detail': traceback.format_exc()
        }), 500

if __name__ == "__main__":
    # 设置上传文件大小限制为100MB
    app.config['MAX_CONTENT_LENGTH'] = 100 * 1024 * 1024
    # 启动服务器，设置监听所有地址，端口5000
    logger.info("启动服务器，监听端口5000")
    app.run(host='0.0.0.0', port=5000, debug=True)